import secp256k1Blake160 from "./secp256k1_blake160";
import secp256k1Blake160Multisig from "./secp256k1_blake160_multisig";
import { MultisigScript, FromInfo, parseFromInfo } from "./from_info";
import dao from "./dao";
import locktimePool, { LocktimeCell } from "./locktime_pool";
import common, { LockScriptInfo, CellCollectorConstructor, CellCollectorType } from "./common";
import sudt from "./sudt";
import anyoneCanPay from "./anyone_can_pay";
import { createP2PKHMessageGroup } from "./p2pkh";
import deploy from "./deploy";
import omnilock from "./omnilock";
export { secp256k1Blake160, secp256k1Blake160Multisig, dao, locktimePool, common, sudt, anyoneCanPay, parseFromInfo, createP2PKHMessageGroup, deploy, omnilock, };
declare const _default: {
    secp256k1Blake160: {
        transfer: typeof import("./secp256k1_blake160").transfer;
        transferCompatible: typeof import("./secp256k1_blake160").transferCompatible;
        payFee: typeof import("./secp256k1_blake160").payFee;
        prepareSigningEntries: typeof import("./secp256k1_blake160").prepareSigningEntries;
        injectCapacity: typeof import("./secp256k1_blake160").injectCapacity;
        setupInputCell: typeof import("./secp256k1_blake160").setupInputCell;
        CellCollector: CellCollectorConstructor;
    };
    secp256k1Blake160Multisig: {
        transfer: typeof import("./secp256k1_blake160_multisig").transfer;
        transferCompatible: typeof import("./secp256k1_blake160_multisig").transferCompatible;
        payFee: typeof import("./secp256k1_blake160_multisig").payFee;
        prepareSigningEntries: typeof import("./secp256k1_blake160_multisig").prepareSigningEntries;
        serializeMultisigScript: typeof import("./from_info").serializeMultisigScript;
        multisigArgs: typeof import("./from_info").multisigArgs;
        injectCapacity: typeof import("./secp256k1_blake160_multisig").injectCapacity;
        setupInputCell: typeof import("./secp256k1_blake160_multisig").setupInputCell;
        CellCollector: CellCollectorConstructor;
    };
    dao: {
        deposit: typeof import("./dao").deposit;
        withdraw: (txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType, fromInput: import("@ckb-lumos/base").Cell, fromInfo?: FromInfo | undefined, { config, enableNonSystemScript }?: import("./dao").WithdrawOptions) => Promise<import("@ckb-lumos/helpers").TransactionSkeletonType>;
        unlock: typeof import("./dao").unlock;
        calculateMaximumWithdraw: typeof import("./dao").calculateMaximumWithdraw;
        calculateMaximumWithdrawCompatible: typeof import("./dao").calculateMaximumWithdrawCompatible;
        calculateDaoEarliestSince: typeof import("./dao").calculateDaoEarliestSince;
        calculateDaoEarliestSinceCompatible: typeof import("./dao").calculateDaoEarliestSinceCompatible;
        CellCollector: typeof import("./dao").CellCollector;
        listDaoCells: typeof import("./dao").listDaoCells;
    };
    locktimePool: {
        CellCollector: CellCollectorConstructor;
        transfer: typeof import("./locktime_pool").transfer;
        transferCompatible: typeof import("./locktime_pool").transferCompatible;
        payFee: typeof import("./locktime_pool").payFee;
        prepareSigningEntries: typeof import("./locktime_pool").prepareSigningEntries;
        injectCapacity: typeof import("./locktime_pool").injectCapacity;
        setupInputCell: typeof import("./locktime_pool").setupInputCell;
        injectCapacityWithoutChange: (txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType, fromInfos: FromInfo[], amount: bigint, tipHeader: import("@ckb-lumos/base").Header, minimalChangeCapacity: bigint, { config, LocktimeCellCollector, enableDeductCapacity, }: {
            config?: import("@ckb-lumos/config-manager/lib").Config | undefined;
            LocktimeCellCollector?: CellCollectorConstructor | undefined;
            enableDeductCapacity?: boolean | undefined;
        }) => Promise<{
            txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType;
            capacity: bigint;
            changeCapacity: bigint;
        }>;
        injectCapacityWithoutChangeCompatible: (txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType, fromInfos: FromInfo[], amount: import("@ckb-lumos/bi").BIish, tipHeader: import("@ckb-lumos/base").Header, minimalChangeCapacity: import("@ckb-lumos/bi").BIish, { config, LocktimeCellCollector, enableDeductCapacity, }: {
            config?: import("@ckb-lumos/config-manager/lib").Config | undefined;
            LocktimeCellCollector?: CellCollectorConstructor | undefined;
            enableDeductCapacity?: boolean | undefined;
        }) => Promise<{
            txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType;
            capacity: import("@ckb-lumos/bi").BI;
            changeCapacity: import("@ckb-lumos/bi").BI;
        }>;
    };
    common: {
        transfer: typeof import("./common").transfer;
        payFee: typeof import("./common").payFee;
        prepareSigningEntries: typeof import("./common").prepareSigningEntries;
        injectCapacity: typeof import("./common").injectCapacity;
        setupInputCell: typeof import("./common").setupInputCell;
        registerCustomLockScriptInfos: typeof import("./common").registerCustomLockScriptInfos;
        payFeeByFeeRate: typeof import("./common").payFeeByFeeRate;
        __tests__: {
            _commonTransfer: (txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType, fromInfos: FromInfo[], amount: bigint, minimalChangeCapacity: bigint, { config, enableDeductCapacity, }?: import("@ckb-lumos/helpers").Options & {
                enableDeductCapacity?: boolean | undefined;
            }) => Promise<{
                txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType;
                capacity: bigint;
                changeCapacity: bigint;
            }>;
            resetLockScriptInfos: () => void;
            getLockScriptInfos: () => {
                configHashCode: number;
                _predefinedInfos: LockScriptInfo[];
                _customInfos: LockScriptInfo[];
                infos: LockScriptInfo[];
            };
            generateLockScriptInfos: ({ config }?: import("@ckb-lumos/helpers").Options) => void;
            getTransactionSizeByTx: (tx: import("@ckb-lumos/base").Transaction) => number;
            getTransactionSize: (txSkeleton: import("@ckb-lumos/helpers").TransactionSkeletonType) => number;
            calculateFee: (size: number, feeRate: bigint) => bigint;
            calculateFeeCompatible: (size: number, feeRate: import("@ckb-lumos/bi").BIish) => import("@ckb-lumos/bi").BI;
        };
    };
    sudt: {
        issueToken: typeof import("./sudt").issueToken;
        transfer: typeof import("./sudt").transfer;
        ownerForSudt: typeof import("./sudt").ownerForSudt;
        packAmount: typeof import("./sudt").packAmount;
        unpackAmount: typeof import("./sudt").unpackAmount;
    };
    anyoneCanPay: {
        CellCollector: CellCollectorConstructor;
        setupInputCell: typeof import("./anyone_can_pay").setupInputCell;
        setupOutputCell: typeof import("./anyone_can_pay").setupOutputCell;
        injectCapacity: typeof import("./anyone_can_pay").injectCapacity;
        prepareSigningEntries: typeof import("./anyone_can_pay").prepareSigningEntries;
        withdraw: typeof import("./anyone_can_pay").withdraw;
    };
};
export default _default;
export type { LocktimeCell, MultisigScript, FromInfo, LockScriptInfo, CellCollectorConstructor, CellCollectorType, };
//# sourceMappingURL=index.d.ts.map